<?php
error_reporting(E_ALL | E_STRICT);
ini_set("track_errors", "On");

$rg_cache_debug = TRUE;

$INC = dirname(__FILE__) . "/../inc";
require_once(dirname(__FILE__) . "/config.php");
require_once($INC . "/init.inc.php");
require_once($INC . "/util.inc.php");
require_once("helpers.inc.php");
require_once("http.inc.php");

rg_log_set_file("http_csrf.log");

$rg_sql = "host=localhost user=rocketgit dbname=rocketgit connect_timeout=10";
$rg_no_db = TRUE;
require_once("common.php");

$_testns = 'http_csrf';
$rg_cache_enable = TRUE;

$now = time();

rg_test_create_user($db, $rg_ui);
$r = test_login($test_url, $rg_ui, $good_sid);
if ($r === FALSE) {
	rg_log("Cannot login!");
	exit(1);
}

rg_log("Loading suggestion form (ua test)");
test_set_ua("user-agent-1");
test_set_referer($test_url);
$data = array();
$headers = array("Cookie: sid=" . $good_sid);
$r = do_req($test_url . "/op/suggestion?t=load_suggestion_form_ua", $data, $headers);
if (!stristr($r['body'], "action=\"/op/suggestion\"")) {
	rg_log_ml(print_r($r, TRUE));
	rg_log("Cannot load form! See above.");
	exit(1);
}
$good_token = $r['token'];

rg_log("Try posting with different user-agent: should not work");
test_set_ua("user-agent-2");
$data = array(
	"doit" => 1,
	"token" => $good_token,
	"suggestion" => "bla bla bla<xss>"
	);
$headers = array("Cookie: sid=" . $good_sid);
$r = do_req($test_url . "/op/suggestion?t=post_suggestion_form_diff_ua", $data, $headers);
if (!stristr($r['body'], "invalid token")) {
	rg_log_ml(print_r($r, TRUE));
	rg_log("Seems I could add a suggestion bypassing CSRF"
		. " protection based on user-agent! See above.");
	exit(1);
}


rg_log("Loading suggestion form (referer test)");
test_set_ua("user-agent-1");
test_set_referer($test_url);
$data = array();
$headers = array("Cookie: sid=" . $good_sid);
$r = do_req($test_url . "/op/suggestion?t=load_suggestion_form_referer", $data, $headers);
if (!stristr($r['body'], "action=\"/op/suggestion\"")) {
	rg_log_ml(print_r($r, TRUE));
	rg_log("Cannot load form! See above.");
	exit(1);
}
$good_token = $r['token'];
$good_logout_token = $r['logout_token'];

rg_log("Try posting with different referer: should not work");
test_set_ua("user-agent-1");
test_set_referer("http://attacker.com:4000/bla");
$data = array(
	"doit" => 1,
	"token" => $good_token,
	"suggestion" => "bla bla bla"
	);
$headers = array("Cookie: sid=" . $good_sid);
$r = do_req($test_url . "/op/suggestion?t=post_suggestion_form_diff_referer", $data, $headers);
if (!stristr($r['body'], "invalid referer")) {
	rg_log_ml(print_r($r, TRUE));
	rg_log_ml("Seems I could add a suggestion bypassing CSRF"
		. " protection based on referer! See above.");
	exit(1);
}


rg_log("Testing logout CSRF (wrong token)...");
test_set_ua("user-agent-1");
test_set_referer($test_url);
$headers = array("Cookie: sid=" . $good_sid);
$data['token'] = strtoupper($good_token);
$r = do_req($test_url . "/op/logout?t=wrong_token", $data, $headers);
if (stristr($r['body'], "You are now logged out")) {
	rg_log_ml(print_r($r, TRUE));
	rg_log("No error on logout with wrong token?! See above.");
	exit(1);
}


rg_log("Testing logout CSRF (token passed in cookie)...");
test_set_ua("user-agent-1");
test_set_referer($test_url);
$headers = array("Cookie: sid=" . $good_sid . "; token=" . $good_logout_token);
$data = array('doit' => 1);
$r = do_req($test_url . "/op/logout?t=token_passed_by_cookie", $data, $headers);
if (stristr($r['body'], "You are now logged out")) {
	rg_log_ml(print_r($r, TRUE));
	rg_log("No error on logout with token passed by cookie?! See above.");
	exit(1);
}


rg_log("Testing logout CSRF (good token)...");
$url = $test_url . "/op/logout?t=good_token&token=" . $good_logout_token;
$data['token'] = $good_token;
$r = do_req($url, $data, $headers);
if (!stristr($r['body'], "You are now logged out")) {
	rg_log_ml(print_r($r, TRUE));
	rg_log("Seems I cannot logout with a good token! See above.");
	exit(1);
}

rg_log("OK!");
?>
